@@ -1,31 +1,17 @@
-# -*- coding:utf-8 -*-
 #
 # Copyright 2014 Hewlett-Packard Development Company, L.P.
 #
-# Licensed under the Apache License, Version 2.0 (the "License"); you may
-# not use this file except in compliance with the License. You may obtain
-# a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-# License for the specific language governing permissions and limitations
-# under the License.
-
+# SPDX-License-Identifier: Apache-2.0
 import ast
 import re
-import sys
 
 import bandit
+from bandit.core import issue
 from bandit.core import test_properties as test
 
-
 RE_WORDS = "(pas+wo?r?d|pass(phrase)?|pwd|token|secrete?)"
 RE_CANDIDATES = re.compile(
-    '(^{0}$|_{0}_|^{0}_|_{0}$)'.format(RE_WORDS),
-    re.IGNORECASE
+    "(^{0}$|_{0}_|^{0}_|_{0}$)".format(RE_WORDS), re.IGNORECASE
 )
 
 
@@ -33,11 +19,13 @@ def _report(value):
     return bandit.Issue(
         severity=bandit.LOW,
         confidence=bandit.MEDIUM,
-        text=("Possible hardcoded password: '%s'" % value))
+        cwe=issue.Cwe.HARD_CODED_PASSWORD,
+        text=f"Possible hardcoded password: '{value}'",
+    )
 
 
-@test.checks('Str')
-@test.test_id('B105')
+@test.checks("Str")
+@test.test_id("B105")
 def hardcoded_password_string(context):
     """**B105: Test for use of hard-coded password strings**
 
@@ -47,6 +35,7 @@ def hardcoded_password_string(context):
 
     - assigned to a variable that looks like a password
     - assigned to a dict key that looks like a password
+    - assigned to a class attribute that looks like a password
     - used in a comparison with a variable that looks like a password
 
     Variables are considered to look like a password if they have match any one
@@ -72,6 +61,7 @@ def hardcoded_password_string(context):
 
         >> Issue: Possible hardcoded password '(root)'
            Severity: Low   Confidence: Low
+           CWE: CWE-259 (https://cwe.mitre.org/data/definitions/259.html)
            Location: ./examples/hardcoded-passwords.py:5
         4 def someFunction2(password):
         5     if password == "root":
@@ -80,9 +70,13 @@ def hardcoded_password_string(context):
     .. seealso::
 
         - https://www.owasp.org/index.php/Use_of_hard-coded_password
+        - https://cwe.mitre.org/data/definitions/259.html
 
     .. versionadded:: 0.9.0
 
+    .. versionchanged:: 1.7.3
+        CWE information added
+
     """
     node = context.node
     if isinstance(node._bandit_parent, ast.Assign):
@@ -90,14 +84,31 @@ def hardcoded_password_string(context):
         for targ in node._bandit_parent.targets:
             if isinstance(targ, ast.Name) and RE_CANDIDATES.search(targ.id):
                 return _report(node.s)
+            elif isinstance(targ, ast.Attribute) and RE_CANDIDATES.search(
+                targ.attr
+            ):
+                return _report(node.s)
 
-    elif (isinstance(node._bandit_parent, ast.Index)
-          and RE_CANDIDATES.search(node.s)):
+    elif isinstance(
+        node._bandit_parent, ast.Subscript
+    ) and RE_CANDIDATES.search(node.s):
+        # Py39+: looks for "dict[candidate]='some_string'"
+        # subscript -> index -> string
+        assign = node._bandit_parent._bandit_parent
+        if isinstance(assign, ast.Assign) and isinstance(
+            assign.value, ast.Str
+        ):
+            return _report(assign.value.s)
+
+    elif isinstance(node._bandit_parent, ast.Index) and RE_CANDIDATES.search(
+        node.s
+    ):
         # looks for "dict[candidate]='some_string'"
         # assign -> subscript -> index -> string
         assign = node._bandit_parent._bandit_parent._bandit_parent
-        if isinstance(assign, ast.Assign) and isinstance(assign.value,
-                                                         ast.Str):
+        if isinstance(assign, ast.Assign) and isinstance(
+            assign.value, ast.Str
+        ):
             return _report(assign.value.s)
 
     elif isinstance(node._bandit_parent, ast.Compare):
@@ -107,10 +118,14 @@ def hardcoded_password_string(context):
             if RE_CANDIDATES.search(comp.left.id):
                 if isinstance(comp.comparators[0], ast.Str):
                     return _report(comp.comparators[0].s)
+        elif isinstance(comp.left, ast.Attribute):
+            if RE_CANDIDATES.search(comp.left.attr):
+                if isinstance(comp.comparators[0], ast.Str):
+                    return _report(comp.comparators[0].s)
 
 
-@test.checks('Call')
-@test.test_id('B106')
+@test.checks("Call")
+@test.test_id("B106")
 def hardcoded_password_funcarg(context):
     """**B106: Test for use of hard-coded password function arguments**
 
@@ -143,6 +158,7 @@ def hardcoded_password_funcarg(context):
         >> Issue: [B106:hardcoded_password_funcarg] Possible hardcoded
         password: 'blerg'
            Severity: Low   Confidence: Medium
+           CWE: CWE-259 (https://cwe.mitre.org/data/definitions/259.html)
            Location: ./examples/hardcoded-passwords.py:16
         15
         16    doLogin(password="blerg")
@@ -150,9 +166,13 @@ def hardcoded_password_funcarg(context):
     .. seealso::
 
         - https://www.owasp.org/index.php/Use_of_hard-coded_password
+        - https://cwe.mitre.org/data/definitions/259.html
 
     .. versionadded:: 0.9.0
 
+    .. versionchanged:: 1.7.3
+        CWE information added
+
     """
     # looks for "function(candidate='some_string')"
     for kw in context.node.keywords:
@@ -160,8 +180,8 @@ def hardcoded_password_funcarg(context):
             return _report(kw.value.s)
 
 
-@test.checks('FunctionDef')
-@test.test_id('B107')
+@test.checks("FunctionDef")
+@test.test_id("B107")
 def hardcoded_password_default(context):
     """**B107: Test for use of hard-coded password argument defaults**
 
@@ -194,6 +214,7 @@ def hardcoded_password_default(context):
         >> Issue: [B107:hardcoded_password_default] Possible hardcoded
         password: 'Admin'
            Severity: Low   Confidence: Medium
+           CWE: CWE-259 (https://cwe.mitre.org/data/definitions/259.html)
            Location: ./examples/hardcoded-passwords.py:1
 
         1    def someFunction(user, password="Admin"):
@@ -202,23 +223,24 @@ def hardcoded_password_default(context):
     .. seealso::
 
         - https://www.owasp.org/index.php/Use_of_hard-coded_password
+        - https://cwe.mitre.org/data/definitions/259.html
 
     .. versionadded:: 0.9.0
 
+    .. versionchanged:: 1.7.3
+        CWE information added
+
     """
     # looks for "def function(candidate='some_string')"
 
     # this pads the list of default values with "None" if nothing is given
-    defs = [None] * (len(context.node.args.args) -
-                     len(context.node.args.defaults))
+    defs = [None] * (
+        len(context.node.args.args) - len(context.node.args.defaults)
+    )
     defs.extend(context.node.args.defaults)
 
     # go through all (param, value)s and look for candidates
     for key, val in zip(context.node.args.args, defs):
-        py3_is_arg = True
-        if sys.version_info.major > 2:
-            py3_is_arg = isinstance(key, ast.arg)
-        if isinstance(key, ast.Name) or py3_is_arg:
-            check = key.arg if sys.version_info.major > 2 else key.id  # Py3
-            if isinstance(val, ast.Str) and RE_CANDIDATES.search(check):
+        if isinstance(key, (ast.Name, ast.arg)):
+            if isinstance(val, ast.Str) and RE_CANDIDATES.search(key.arg):
                 return _report(val.s)
