@@ -1,19 +1,7 @@
-# -*- coding:utf-8 -*-
 #
 # Copyright 2014 Hewlett-Packard Development Company, L.P.
 #
-# Licensed under the Apache License, Version 2.0 (the "License"); you may
-# not use this file except in compliance with the License. You may obtain
-# a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-# License for the specific language governing permissions and limitations
-# under the License.
-
+# SPDX-License-Identifier: Apache-2.0
 r"""
 ============================
 B608: Test for SQL injection
@@ -39,6 +27,13 @@ If so, a MEDIUM issue is reported. For example:
 
  - cursor.execute("SELECT %s FROM derp;" % var)
 
+Use of str.replace in the string construction can also be dangerous.
+For example:
+
+- "SELECT * FROM foo WHERE id = '[VALUE]'".replace("[VALUE]", identifier)
+
+However, such cases are always reported with LOW confidence to compensate
+for false positives, since valid uses of str.replace can be common.
 
 :Example:
 
@@ -47,7 +42,8 @@ If so, a MEDIUM issue is reported. For example:
     >> Issue: Possible SQL injection vector through string-based query
     construction.
        Severity: Medium   Confidence: Low
-       Location: ./examples/sql_statements_without_sql_alchemy.py:4
+       CWE: CWE-89 (https://cwe.mitre.org/data/definitions/89.html)
+       Location: ./examples/sql_statements.py:4
     3 query = "DELETE FROM foo WHERE id = '%s'" % identifier
     4 query = "UPDATE foo SET value = 'b' WHERE id = '%s'" % identifier
     5
@@ -55,24 +51,31 @@ If so, a MEDIUM issue is reported. For example:
 .. seealso::
 
  - https://www.owasp.org/index.php/SQL_Injection
- - https://security.openstack.org/guidelines/dg_parameterize-database-queries.html  # noqa
+ - https://security.openstack.org/guidelines/dg_parameterize-database-queries.html
+ - https://cwe.mitre.org/data/definitions/89.html
 
 .. versionadded:: 0.9.0
 
-"""
+.. versionchanged:: 1.7.3
+    CWE information added
 
+.. versionchanged:: 1.7.7
+    Flag when str.replace is used in the string construction
+
+"""  # noqa: E501
 import ast
 import re
 
 import bandit
+from bandit.core import issue
 from bandit.core import test_properties as test
 from bandit.core import utils
 
 SIMPLE_SQL_RE = re.compile(
-    r'(select\s.*from\s|'
-    r'delete\s+from\s|'
-    r'insert\s+into\s.*values\s|'
-    r'update\s.*set\s)',
+    r"(select\s.*from\s|"
+    r"delete\s+from\s|"
+    r"insert\s+into\s.*values\s|"
+    r"update\s.*set\s)",
     re.IGNORECASE | re.DOTALL,
 )
 
@@ -83,38 +86,58 @@ def _check_string(data):
 
 def _evaluate_ast(node):
     wrapper = None
-    statement = ''
+    statement = ""
+    str_replace = False
 
     if isinstance(node._bandit_parent, ast.BinOp):
         out = utils.concat_string(node, node._bandit_parent)
         wrapper = out[0]._bandit_parent
         statement = out[1]
-    elif (isinstance(node._bandit_parent, ast.Attribute)
-          and node._bandit_parent.attr == 'format'):
+    elif isinstance(
+        node._bandit_parent, ast.Attribute
+    ) and node._bandit_parent.attr in ("format", "replace"):
         statement = node.s
         # Hierarchy for "".format() is Wrapper -> Call -> Attribute -> Str
         wrapper = node._bandit_parent._bandit_parent._bandit_parent
-    elif (hasattr(ast, 'JoinedStr')
-          and isinstance(node._bandit_parent, ast.JoinedStr)):
-        statement = node.s
-        wrapper = node._bandit_parent._bandit_parent
+        if node._bandit_parent.attr == "replace":
+            str_replace = True
+    elif hasattr(ast, "JoinedStr") and isinstance(
+        node._bandit_parent, ast.JoinedStr
+    ):
+        substrings = [
+            child
+            for child in node._bandit_parent.values
+            if isinstance(child, ast.Str)
+        ]
+        # JoinedStr consists of list of Constant and FormattedValue
+        # instances. Let's perform one test for the whole string
+        # and abandon all parts except the first one to raise one
+        # failed test instead of many for the same SQL statement.
+        if substrings and node == substrings[0]:
+            statement = "".join([str(child.s) for child in substrings])
+            wrapper = node._bandit_parent._bandit_parent
 
     if isinstance(wrapper, ast.Call):  # wrapped in "execute" call?
-        names = ['execute', 'executemany']
+        names = ["execute", "executemany"]
         name = utils.get_called_name(wrapper)
-        return (name in names, statement)
+        return (name in names, statement, str_replace)
     else:
-        return (False, statement)
+        return (False, statement, str_replace)
 
 
-@test.checks('Str')
-@test.test_id('B608')
+@test.checks("Str")
+@test.test_id("B608")
 def hardcoded_sql_expressions(context):
-    val = _evaluate_ast(context.node)
-    if _check_string(val[1]):
+    execute_call, statement, str_replace = _evaluate_ast(context.node)
+    if _check_string(statement):
         return bandit.Issue(
             severity=bandit.MEDIUM,
-            confidence=bandit.MEDIUM if val[0] else bandit.LOW,
+            confidence=(
+                bandit.MEDIUM
+                if execute_call and not str_replace
+                else bandit.LOW
+            ),
+            cwe=issue.Cwe.SQL_INJECTION,
             text="Possible SQL injection vector through string-based "
-                 "query construction."
+            "query construction.",
         )
