@@ -11,3 +11,5 @@ os.popen2('/bin/uname -av')
 os.popen3('/bin/uname -av')
 os.popen4('/bin/uname -av')
 
+os.popen4('/bin/uname -av; rm -rf /')
+os.popen4(some_var)
