@@ -9,5 +9,5 @@ pop(['/bin/ls', '-l'], shell=False)
 
 pop('../ls -l', shell=False)
 
-pop('c:\hello\something', shell=False)
+pop('c:\\hello\\something', shell=False)
 pop('c:/hello/something_else', shell=False)
